// function loader() {
//     $.getScript("data/game.js", function(){
//         alert("Script loaded and executed.");
//     });
// }

function cuzgame_init() {
    var G = {};
    G.params = {};
    G.images = {};
    G.files = {};
    G.stream = null;
    G.fonts = {};
    G.samples = {};
    
    var NIL = [];

    function _preloadAudio(name) {
        if (G.samples[name] == null) {
            var url = G.dataurl + name + _audioExt();
            var audio = new Audio(url);
            audio.load();
            G.samples[name] = audio;
        }
        return G.samples[name];
    }

    function _preloadImage(name) {
        if (G.images[name] == null) {
            var url = G.dataurl + name + ".png";
            var img = new Image();
            img.src = url;
            G.images[name] = img;
        }
        return G.images[name];
    }

    
    G.api = {
        
'cuz_params_get': function(k) {
    return [G.params[k]];
},

'cuz_params_set': function (k,v) {
    G.params[k] = v;
    return NIL;
},

'cuz_file_load': function (name) {
    return [G.files[name]];
},

'cuz_file_save': function (name,data) {
    G.files[name] = data;
    return NIL;
},

'cuz_view_set': function (x,y,w,h,depth) {
    G.view = {x:x,y:y,w:w,h:h};
    return NIL;
},

'cuz_image_draw': function (name,x,y,w,h,cx,cy,angle,color,alpha,mode,depth) {

    x = (x - G.view.x) * 720 / G.view.w;
    y = (y - G.view.y) * 480 / G.view.h;
    w = w * 720 / G.view.w;
    h = h * 480 / G.view.h;
    
    var img = _preloadImage(name);
    
    if (w<0) { x = x-w; }
    if (h<0) { y = y-h; }
    
    if (name == "white" && color != null) {
        _drawFill(color,x,y,w,h);
        return NIL;
    }
    
    _drawImage(img,0,0,img.width,img.height,x,y,w,h);
    
    return NIL;
},

'cuz_sample_play': function (name,vol,pan) {
    if (G.mute) { return NIL; }
    if (vol == null) { vol = 1.0; }
    if (pan == null) { pan = 0.0; }
    
//     var sample = createjs.Sound.play(name);
//     sample.volume = vol;
    
    // very simple raw JS
//     var url = G.dataurl + name + _audioExt()
//     var sample = new Audio(url);
    var sample = _preloadAudio(name);
    sample.pause();
    sample.currentTime = 0;
    sample.play();
    return NIL;
},

'cuz_stream_play': function (name,vol,pan) {
    if (G.mute) { return NIL; }
    if (vol == null) { vol = 1.0; }
    if (pan == null) { pan = 0.0; }
    G.cuz_stream_stop();
    
//     var stream = createjs.Sound.play(name);
//     console.log(vol);
//     stream.volume = vol;
//     G.stream = stream;
    
    // very simple raw JS
//     var url = G.dataurl + name + _audioExt()
//     var stream = new Audio(url);
    var stream = _preloadAudio(name);
    stream.play();
    stream.loop = true;
    G.stream = stream;
    
    return NIL;
},

'cuz_stream_stop': function (name) {
    if (G.stream == null) { return NIL; }
    
//     G.stream.stop();
   
    // very simple raw JS
    stream.pause();
    stream.currentTime = 0;
    G.stream = null;
    return NIL;
},

'_cuz_font_load': function (name,points) {
    if (G.fonts[name] == null) {
    
        var font = { chars:{} };
        G.fonts[name] = font;
        font.image = new Image();
        var url = G.dataurl + name + ".png";
        font.image.src = url;
        var data = G.files[name+".fnt"];
        var lines = data.split("\n");
        for (lines_k in lines) {
            var line = lines[lines_k];
            var parts = line.split(" ");
            var e = {};
            for (parts_k in parts) {
                var pp = parts[parts_k].split("=");
                if (pp.length < 2) { continue; }
                e[pp[0]]=pp[1];
            }
            if (parts[0] == "common") {
                font.height = parseFloat(e.lineHeight);
                font.base = parseFloat(e.base);
                font.rect = {x:0,y:0,w:parseFloat(e.scaleW),h:parseFloat(e.scaleH)};
                font._size = font.height;
                font.size = font._size;
                font.scale = 1.0;
            }
            if (parts[0] == "char") {
                var n = parseInt(e.id);
                if (n <= 0) { continue; }
                var chr = {};
                chr.rect = {x:parseFloat(e.x),y:parseFloat(e.y),w:parseFloat(e.width),h:parseFloat(e.height)};
                chr.shape = {x:parseFloat(e.xoffset),y:parseFloat(e.yoffset),w:parseFloat(e.xadvance),h:parseFloat(e.height)};
                font.chars[n] = chr;
//                 console.log(font.chars[n],n,chr);
            }
        }
    }
    
    var font = G.fonts[name];
    font.size = (points ? points : font._size);
    font.scale = font.size / font._size;
    
    return font;
},
    

'cuz_font_draw': function (name,points,text,x,y,w,align,color,alpha,mode) {
    var font = G._cuz_font_load(name,points);
    var img = font.image;
    x = x * 720 / G.view.w;
    y = y * 480 / G.view.h;
    var scale = font.scale * 720 / G.view.w;
//     console.log(font,img,scale);
//     return;
    for (var i=0; i<text.length; i++) {
        var n = parseInt(text.charCodeAt(i));
        var chr = font.chars[n];
        
        if (chr == null) { continue; }
        var cx = chr.rect.x;
        var cy = chr.rect.y;
        var cw = chr.rect.w;
        var ch = chr.rect.h;
//         console.log(n,font.chars[n],ch,ch.rect,ch.shape);
        var dx = x + chr.shape.x * scale;
        var dy = y + chr.shape.y * scale;
        var dw = chr.rect.w * scale;
        var dh = chr.rect.h * scale;
//         console.log(img,cx,cy,cw,ch,dx,dy,dw,dh);
        _drawImage(img,cx,cy,cw,ch,dx,dy,dw,dh);
        x = x + chr.shape.w * scale;
    }
    return NIL;
},

'cuz_font_size': function (name,points,text,ww) {
    var font = G._cuz_font_load(name,points);
    var w = 0;
    var h = font.size;
    for (var i=0; i<text.length; i++) {
        var n = parseInt(text.charCodeAt(i));
        var chr = font.chars[n];
        if (chr == null) { continue; }
        w = w + chr.shape.w * font.scale;
    }
    return [w,h];
},

'cuz_ext_call': function(name,data) {
    if (name == 'open_url') { window.open(data,"_blank"); }
    return NIL;
},

'json_decode': function(v) {
    var data = eval(v);
    var r = _tolua(data);
    return [r];
},

    };

    
    for (var k in G.api) {
        G[k] = G.api[k];
    }
    
    return G;
}
    
CUZGAME = cuzgame_init();



function _tolua(e) {
    if (typeof(e) != "object") { return e; }
    var r = lua_newtable(null);
    for (k in e) {
        var key = k;
        if (!isNaN(parseInt(key))) { key = parseInt(key)+1; }
        lua_tableset(r,key,_tolua(e[k]));
    }
    return r;
}
function _sign(v) {
    return (v>=0?1:-1);
}

function _drawImage(img,cx,cy,cw,ch,x,y,w,h) {
    if (cw == 0 || ch == 0 || w == 0 || h == 0) { return; }
    
    var G = CUZGAME;
    if (w >= 0 && h >= 0) {
//         console.log(img,cx,cy,cw,ch,x,y,w,h);
        G.context.drawImage(img,cx,cy,cw,ch,x,y,w,h);
        return;
    }
    G.context.save();
    G.context.scale(_sign(w),_sign(h));
    if (w < 0) { x = -x; w = -w; }
    if (h < 0) { y = -y; h = -h; }
    G.context.drawImage(img,cx,cy,cw,ch,x,y,w,h);
    G.context.restore();
}

function _drawFill(color,x,y,w,h) {
    var G = CUZGAME;
    color = color.toString(16);
    while (color.length < 6) { color = "0" + color; }
    G.context.fillStyle = "#"+color;
    G.context.fillRect(x,y,w,h);
}

function _audioExt() {
//     return ".mp3";
    var audio = new Audio();
    if (audio.canPlayType != null && audio.canPlayType('audio/ogg; codecs="vorbis"') != "") {
        return ".ogg"; 
    }
    return ".mp3";
}

var luajs_engine = {
    load: function() {
        var G = CUZGAME;
        for (var k in G.api) {
            lua_core[k] = G.api[k];
        }
        lua_script = lua_script_load()[0];
//         lua_script = lua_load(G.files["script.txt"],"script");
    },
    init: function() {
        lua_call(lua_tableget(lua_script, "init"), []);
    },
    event: function(e) {
        var event = lua_newtable(null);
        for (var k in e) {
            lua_tableset(event,k,e[k]);
        }
        lua_call(lua_tableget(lua_script, "event"), [event]);
    },
    loop: function(t) {
        lua_call(lua_tableget(lua_script, "loop"), [t]);
    },
    paint: function() {
        lua_call(lua_tableget(lua_script, "paint"), []);
    },
};

var luavmjs_engine = {
    load: function() {
        var G = CUZGAME;
        delete G.api.json_decode ;
        for (var k in G.api) {
            var args = G.api[k].toString();
            args = args.substring(args.indexOf('(')+1,args.indexOf(')'))
            var tmp = "function "+k+"("+args+") " +
                "local r = js.global.CUZGAME."+k+"("+args+"); "+
                "return unpack(r) end"
//             console.log(tmp);
            Lua.execute(tmp);
        }
        Lua.execute(G.files["script.txt"]);
    },
    init: function() {
        Lua.execute("init()");
    },
    event: function(e) {
        var tmp = "event({";
        for (k in e) {
            var v = e[k];
            if (typeof(v) == "number") { tmp = tmp + k + "=" + e[k] + "," }
            else { tmp = tmp + k + "=\"" + e[k] + "\"," }
        }
        tmp = tmp + "})";
        Lua.execute(tmp);
    },
    loop: function(t) {
        Lua.execute("loop("+t+")");
    },
    paint: function() {
        Lua.execute("paint()");
    },
};

function _lua51js_error() {
    var G = CUZGAME; var C = Lua5_1.C; var L = CUZGAME.L;
    var err = C.lua_tostring(L, -1);
    C.lua_close(L);
    G.L = null;
    throw new Error("Lua error: " + err);
}

function _lua51js_dostring(code) {
    var G = CUZGAME; var C = Lua5_1.C; var L = CUZGAME.L;
    if (G.L == null) { return; }
    if (C.luaL_dostring(L,code) != 0) {
        _lua51js_error();
    }
}

function _lua51js_function(k) {
    var G = CUZGAME; var C = Lua5_1.C; var L = CUZGAME.L;
    var fnc = Lua5_1.Runtime.addFunction(
        function (L) {
            var argc = C.lua_gettop(L);
            var args = [];
            for (var n=1; n<=argc; n++) {
                if (C.lua_type(L,n) == C.LUA_TNUMBER) {
                    args.push(C.lua_tonumber(L,n));
                } else {
                    args.push(C.lua_tostring(L,n));
                }
            }
//             console.log(k,args);
            var r = G[k].apply(null,args);
            for (var n=0; n<r.length; n++) {
                var v = r[n];
                if (typeof(v) == "number") {
                    C.lua_pushnumber(L,v);
                } else {
                    C.lua_pushstring(L,v);
                }
            }
            return r.length;
        }
    );
    return fnc;
}

var lua51js_engine = {
    load: function() {
        CUZGAME.L = Lua5_1.C.lua_open();
        var G = CUZGAME; var C = Lua5_1.C; var L = CUZGAME.L;
        C.luaL_openlibs(L);
        delete G.api.json_decode ;
        for (var k in G.api) {
            var fnc = _lua51js_function(k);
            C.lua_pushcfunction(L,fnc);
            C.lua_setglobal(L, k);
        }
        _lua51js_dostring(G.files["script.txt"]);
    },
    init: function() {
        _lua51js_dostring("init()");
    },
    event: function(e) {
        var tmp = "event({";
        for (k in e) {
            var v = e[k];
            if (typeof(v) == "number") { tmp = tmp + k + "=" + v + "," }
            else { tmp = tmp + k + "=\"" + v + "\"," }
        }
        tmp = tmp + "})";
//         console.log(tmp);
        _lua51js_dostring(tmp);
    },
    loop: function(t) {
        _lua51js_dostring("loop("+t+")");
    },
    paint: function() {
        var G = CUZGAME; var C = Lua5_1.C; var L = CUZGAME.L;
        _lua51js_dostring("paint()");
//         C.lua_getglobal(L,"paint");
//         if (C.lua_pcall(L,0,0,0) != 0) {
//             _lua51js_error();
//         }
    },
};

function init() {
    var G = CUZGAME;
    preload_init(G);
    
    G.canvas = document.getElementById("canvas");
    G.context = G.canvas.getContext("2d");
    G.dataurl = "data/";
    G.fps = 30;
    G.mute = false;
    
    
    G.script = luajs_engine; // https://github.com/mherkender/lua.js - works!
//     G.script = luavmjs_engine; // https://github.com/kripken/lua.vm.js - horribly buggy
//     G.script = lua51js_engine; // https://github.com/logiceditor-com/lua5.1.js
    G.script.load();
    G.script.init();
    
    window.addEventListener("keydown", doKeyDown, false);
    window.addEventListener("keyup", doKeyUp, false);

//     G.canvas.addEventListener("mousedown", doMouseDown, false);
//     G.canvas.addEventListener("mouseup", doMouseUp, false);
//     G.canvas.addEventListener("mousemove",doMouseMove, false);
    $("canvas").mousedown(doMouseDown);
    $("canvas").mouseup(doMouseUp);
    $("canvas").mousemove(doMouseMove);
    
    // if no interval is set, we run at max framerate
    G.interval = setInterval("tick("+(1.0/G.fps)+");", 1000/G.fps);
//     if (G.interval == null) { tick(); }
}

function getKeyName(e) {
    var name = "?"
    if (e.keyCode == 38) { name = "Up"; }
    if (e.keyCode == 39) { name = "Right"; }
    if (e.keyCode == 40) { name = "Down"; }
    if (e.keyCode == 37) { name = "Left"; }
    if (e.keyCode == 32) { name = "Space"; }
    if (e.keyCode == 27) { name = "Escape"; }
    if (e.keyCode == 13) { name = "Return"; }
    if (e.keyCode == 9) { name = "Tab"; }
    if (name == "?") { name = String.fromCharCode(e.keyCode); }
    return name;
}

function doKeyDown(e) {
//     console.log("doKeyDown:"+e.keyCode+":"+getKeyName(e));
    event({type:"keydown",name:getKeyName(e),key:e.keyCode});
}
function doKeyUp(e) {
//     console.log("doKeyUp:"+e.keyCode+":"+getKeyName(e));
    event({type:"keyup",name:getKeyName(e),key:e.keyCode});
}


function doMouseDown(e) {
    var G = CUZGAME;
    var x = e.pageX - $(this).position().left;
    var y = e.pageY - $(this).position().top;
    var scale = G.view.w / 720;
    event({type:"down",x:x*scale,y:y*scale});
}
function doMouseUp(e) {
    var G = CUZGAME;
    var x = e.pageX - $(this).position().left;
    var y = e.pageY - $(this).position().top;
    var scale = G.view.w / 720;
    event({type:"up",x:x*scale,y:y*scale});
}
function doMouseMove(e) {
    var G = CUZGAME;
    var x = e.pageX - $(this).position().left;
    var y = e.pageY - $(this).position().top;
    var scale = G.view.w / 720;
    event({type:"motion",x:x*scale,y:y*scale});
}

function track_fps() {
    var G = CUZGAME;
    if (G.track_fps == null) {
        G.track_fps = {prev:new Date().getTime(),ticks:1000,fps:0,frame:0};
    }
    G = G.track_fps;

    var cur = new Date().getTime();
    var delta = (cur-G.prev);
    G.prev = cur;
    var r = 0.95;
    G.ticks = G.ticks * r + delta * (1.0-r);
    if ((G.frame%100)==0) {
        G.fps = Math.round(1000.0 / G.ticks);
        console.log('track_fps',G.fps);
    }
    G.frame += 1;
}

function event(e) {
    var G = CUZGAME;
    track_time_begin("event");
    G.script.event(e);
    track_time_end("event");
}


var _track_time = {};
function track_time_begin(ctx) {
    if (_track_time[ctx] == null) { 
        _track_time[ctx] = {name:ctx,total:0,hits:0,ts:0};
    }
    var c = _track_time[ctx];
    c.ts = new Date().getTime();
}
function track_time_end(ctx) {
    var c = _track_time[ctx];
    c.total += new Date().getTime() - c.ts;
    c.hits += 1;
    if ((c.hits%100)==0) {
//         console.log(ctx,c.total);
        c.total=0;
    }
}
    
function loop(t) {
    var G = CUZGAME;
    track_time_begin("loop");
    G.script.loop(t);
    track_time_end("loop");
}

function paint() {
    var G = CUZGAME;
    if (G.params.filter == "nearest") {
        G.context.imageSmoothingEnabled = false;
        G.context.mozImageSmoothingEnabled = false;
        G.context.webkitImageSmoothingEnabled = false;
    }
    G.context.fillStyle = "black";
    G.context.fillRect(0,0,720,480);
    track_time_begin("paint");
    G.script.paint();
    track_time_end("paint");
}    

function tick() {
    var G = CUZGAME;
    var cur = new Date().getTime();
    if (G.t == null) { G.t = cur; }
    var t = Math.min(1.0/G.fps,(cur - G.t) / 1000.0);
//     console.log(t*1000);
    G.t = cur
    loop(t);
    paint();
//     track_fps();
    
    if (G.interval == null) {
        setTimeout("tick()",0);
    }
}

